######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  class DDYFile
    def initialize(path)
      @path = path
      @lines = []
      if File.exists?(@path)
        File.open(@path) do |f|
          while line = f.gets
            @lines << line.chomp
          end
        end
      end
    end

    def objects
      objects = []
      object_fields = []
      object_type = ''
      inobject = false
      object_comment = nil
      @lines.each_index do |i|
        if not inobject and @lines[i] =~ /^\s*\w/
          object_fields = []
          inobject = true
          object_type = @lines[i]
          if @lines[i-1] =~ /^\s*!/
            object_comment = @lines[i-1]
          end
        elsif inobject and @lines[i] =~ /^[^!]*;/
          object_fields << @lines[i]
          inobject = false
          objects << IdfObject.new(object_type,object_fields,object_comment)
        elsif inobject
          object_fields << @lines[i]
        else
        end
      end
      return objects
    end

    def findObjectByComment(comment)
      comment = Regexp.new(comment)
      objects().each do |object|
        if object.comment =~ comment
          return object
        end
      end

      return nil
    end

    #returns an array of "data source, year, and type
    def get_data_source
      #! Using Design Conditions from "Climate Design Data 2005 ASHRAE Handbook"
      #! Using Design Conditions from "Climate Design Data 2009 ASHRAE Handbook"
      src_s = ""
      @lines.each_index do |i|
        if @lines[i] =~ /Climate Design Data [0-9]{4} ASHRAE Handbook/
          src_s = @lines[i]
          break
        end
      end

      src = []
      src << src_s.match("Climate Design Data [0-9]{4} ASHRAE Handbook")[0]
      src << src_s.match("[0-9]{4}")[0]
      src << "ASHRAE Handbook"

      return src

    end

    def write_idf_snippet(filename, ddyobj)
      if not ddyobj.nil?
        File.delete(filename) if File.exists?(filename)
        File.open(filename, 'w') do |fl|
      #add version identifier to file
        fl << "\nVersion,\n  7.1;                                    ! Version Identifier \n\n"
          fl << ddyobj
        end
      end
    end

    #updated for ddy for EP 7.1 2012
    def get_indicating_type(ddyobj)
      val = ddyobj.fieldString(8).downcase
      if val == "dewpoint"
      res = []
      res << val
      res << ddyobj.fieldValue(9)
      res << "C"
      elsif val == "wetbulb"
      res = []
      res << val
      res << ddyobj.fieldValue(9)
        res << "C"
     elsif val == "enthalpy"
      res = []
      res << val
      res << ddyobj.fieldValue(12)
      res << "kJ/kg"
      end
      return res
    end

    #untyped enumerations for heating, cooling, and humidification
    #make this an array, the first column of the array is the ep object,
    #the remaining items are the "attributes and tags"
    def heating(typestr)
      search = "Annual Heating " + typestr

      obj = findObjectByComment(search)

      puts "[DDYFile] #{File.basename(@path.to_s)} missing heating #{typestr}" if obj.nil?

      ddyinfo = []
      ddyinfo << search.gsub("\\","")
      ddyinfo << search.gsub("\\","").gsub(" ","_").gsub(".","_").gsub("%","").gsub("=>","_").gsub("__","_")
      ddyinfo << obj

      return ddyinfo
    end

    def humidification(typestr)
      search = "Annual Humidification " + typestr

      obj = findObjectByComment(search)

      puts "[DDYFile] #{File.basename(@path.to_s)} missing humidification #{typestr}" if obj.nil?

      ddyinfo = []
      ddyinfo << search.gsub("\\","")
      ddyinfo << search.gsub("\\","").gsub(" ","_").gsub(".","_").gsub("%","").gsub("=>","_").gsub("__","_")
      ddyinfo << obj

      return ddyinfo
    end

    def cooling(typestr)
      search = "Annual Cooling " + typestr

      obj = findObjectByComment(search)

      puts "[DDYFile] #{File.basename(@path.to_s)} missing cooling #{typestr}" if obj.nil?

      ddyinfo = []
      ddyinfo << search.gsub("\\","")
      ddyinfo << search.gsub("\\","").gsub(" ","_").gsub(".","_").gsub("%","").gsub("=>","_").gsub("__","_")
      ddyinfo << obj

      return ddyinfo

    end
  end

end #moduel