######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  class EpErrFile

    def initialize(path)
      @path = path.expand_path
      @warnings = []
      @severe_errors = []
      @fatal_errors = []
      @initialized = false
      @successful = false
    end

    def init
      @initialized = true

      if @path.exist?
        File.open(@path) do |f|
          text = ''
          text << f.read
          @valid = true
          @warnings = parse("Warning", text)
          @severe_errors = parse("Severe", text)
          @fatal_errors = parse("Fatal", text)
          if text =~ /\s\*{13} EnergyPlus Completed Successfully/
            @successful = true
          end
        end
      end
    end

    def valid?
      init if not initialized?
      @valid
    end

    def initialized?
      @initialized
    end

    def path
      @path
    end

    def dirname
      @path.dirname
    end

    def warnings
      init if not initialized?
      @warnings
    end

    def severe_errors
      init if not initialized?
      @severe_errors
    end

    def fatal_errors
      init if not initialized?
      @fatal_errors
    end

    # just a warning
    def warning_count
      init if not initialized?
      warnings.size
    end

    # severe errors are not as bad as fatal
    def severe_error_count
        init if not initialized?
        @severe_errors.size
    end

    # the worst error
    def fatal_error_count
        init if not initialized?
        @fatal_errors.size
    end

    # work horse
    def parse(id, text)
      storage = []
      this_text = text
      # match first id label and continue matching across lines while seeing the continue label "**   ~~~   **"
      regex = /\*\*\s*#{id}\s*\*\*.*?\n(\s+\*\*\s\s\s~~~\s\s\s\*\*.*?\n)*/m
      while match_data = this_text.match(regex)
        this_string = match_data.to_s.gsub(/.*\*\*\s*#{id}\s*\*\*/, '').gsub(/.*\*\*\s\s\s~~~\s\s\s\*\*/, '').gsub(/\s*\n/, "\n")
        storage.push(this_string)
        this_text = match_data.post_match
      end
      return storage.uniq
    end

    def successful?
      init if not initialized?
      @successful
    end


  end

end #module