######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  class EpImfFile
    attr_accessor :locations

    def initialize(path)
      puts 'parsing ' + path
      @path = path.expand_path
      @text = ''
      @locations = []

      if @path.exist?
        File.open(@path) do |f|
          @text << f.read
          parse
        end
      end
    end

    def savefileas(path)
      File.open(path, "w") do |f|
        @locations.each do |loc|
          f << "##def " + loc[0] + "[]\n"
          f << loc[1]
          f << "##enddef\n\n"
      end
      end

    end

    def parse
      this_text = @text

      locations = Hash.new

      regex = /##def.*?##enddef/im
      while match_data = this_text.match(regex)
        #Grab the name of the location
        regex2 = /##def (.*?)\[\]$/im

        location_name = match_data.to_s.match(regex2)[1].lstrip.rstrip

        #remove the header/footer from imf snippet
        regex3 = /##def (.*?)\[\]\n*/im
        regex4 = /##enddef(.*)\n*/im
        imf_data = match_data.to_s.gsub(regex3, "").gsub(regex4, "")

        #clean up the IMF (which is really idf data at this point)
        imf_data.gsub!(/^(  )/,"")

        location = [location_name, imf_data]
        @locations.push(location)

        this_text = match_data.post_match
      end
    end

    def location_sort(x,y)
      if x[0].include?('ShowHeader')
        return -1
      elsif y[0].include?('ShowHeader')
        return 1
      else
        return x[0] <=> y[0]
      end
    end

    def sort
      @locations.sort! {|x,y| location_sort(x,y)}
    end

  end

end #module