######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

require 'pathname'
require 'csv'

module EnergyPlus

  class EpwFile
    attr_accessor :path
    attr_accessor :city
    attr_accessor :state
    attr_accessor :country
    attr_accessor :typ
    attr_accessor :wmo
    attr_accessor :lat
    attr_accessor :lon
    attr_accessor :gmt
    attr_accessor :elevation
    attr_accessor :url

    def initialize(path)
      @path = Pathname.new(path)
      @city = ""
      @state = ""
      @country = ""
      @typ = ""
      @wmo = ""
      @lat = ""
      @lon = ""
      @gmt = ""
      @elevation = ""
      @url = ""
      init
    end

    def toKml(xml)
      xml.Placemark {
        xml.name @city
        xml.visibility "0"
        xml.description {
          xml.cdata!("<img src=\"kml/ep_header8.png\" width=180 align=right><br><table><tr><td colspan=\"2\">"+
                     "<b>#{@city}</b></href></td></tr>\n"+
           #"<tr><td></td><td><b>Data Type</td></tr>\n"+
           "<tr><td></td><td>WMO <b>#{@wmo}</b></td></tr>\n"+
           #"<tr><td></td><td>E   3� 15'   N 36� 43'</td></tr>\n"+
         #"<tr><td></td><td><b>25</b> m</td></tr>\n"+
         "<tr><td></td><td>Time Zone GMT <b>#{@gmt}</b> hours</td></tr>\n"+
         #"<tr><td></td><td>ASHRAE Std 169 Climate Zone <b>4A - Mixed - Humid</b></td></tr>\n"+
         #"<tr><td></td><td>99% Heating DB=<b>3.1</b>, 1% Cooling DB=<b>33.2</b></td></tr>\n"+
         #"<tr><td></td><td>HDD18 <b>1019</b>, CDD10 <b>2849</b></td></tr>\n"+
         "<tr><td></td><td>URL #{@url}</td></tr></table>")
        }
        xml.styleUrl "#weatherlocation"
        xml.Point {
          xml.altitudeMode "absolute"
          xml.coordinates "#{@lon},#{@lat},#{elevation}"
        }
      }
    end

    def valid?
      return @valid
    end



    # the mean of the mean monthly dry bulbs
    private

    # initialize
    def init
      if @path.exist?
        rowCount = 0
        CSV.open(@path, 'r') do |row|
          rowCount += 1

          # LOCATION,Adak Nas,AK,USA,TMY3,704540,51.88,-176.65,-10.0,5.0
          @valid = true

          # process only header row
          if rowCount == 1
            @city = row[1].gsub("/","-")
            @state = row[2]
            @country = row[3]
            @typ = row[4]
            @wmo = row[5]
            @wmo = "wmoundefined" if @wmo.nil?
            @lat = row[6]
            @lon = row[7]
            @gmt = row[8]
            @elevation = row[9]
            break
          end


        end
      end
    end


  end

end #module