######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  class DictionaryEntry
    attr_reader :id, :key, :var_name, :units, :interval
    def initialize(id,key,var_name,units,interval)
      @id,@key,@var_name,@units,@interval = id,key,var_name,units,interval
    end
  end

  class Variable
  #	attr_reader :data
    def initialize(dictionary_entry,data,timestamps)
      @dictionary_entry = dictionary_entry
      @data = data
      @timestamps = timestamps
    end

    def name
      return @dictionary_entry.var_name
    end

    def key
      return @dictionary_entry.key
    end

    def id
      return @dictionary_entry.id
    end

    def units
      return @dictionary_entry.units
    end

    def interval
      return @dictionary_entry.interval
    end

    def values
      @data.map { |entry| entry[1] }
    end

    def hours
      @timestamps.map { |entry| entry[5] }
    end

    def days
      @timestamps.map { |entry| entry[1] }
    end

    def months
      @timestamps.map { |entry| entry[2] }
    end
  end

  class EpEsoFile
    def initialize(path)
      File.open(path) do |f|
        @variables = nil
        # first six lines are discarded
        6.times {f.gets}
        @text = f.read
      end
    end

    def dictionary
      return @text.match(/(.*?)(End\sof\sData\sDictionary)/m)[1]
    end

    def data
      return @text.match(/(End\sof\sData\sDictionary)(.*?)(End\sof\sData)/m)[2]
    end

    def dictionary_entries
      dictionary_entries = []
      dictionary.split("\n").each do |entry|
        interval = entry.match(/(!)(\w+)(\s*)/)[2].strip
        fields = entry.match(/(.*?)(!)/)[1].split(',')
        id = fields[0]
        key,var_name = fields[2,3] if fields[1] =~ /2/
        key,var_name = nil, fields[2] if fields[1] =~ /1/
        units = var_name.match(/(.*?)(\[)(.*?)(\])/)[3]
        var_name = $1.strip
        dictionary_entries << DictionaryEntry.new(id,key,var_name,units,interval)
      end
      return dictionary_entries
    end

    def variables
      unless @variables
        @variables = []
        text = data
        dictionary_entries.each do |dictionary_entry|
          if dictionary_entry.interval =~ /Hourly/
            timestamps = text.scan(/^2,.*?,\s0\.00,60\.00,.*/).map { |entry| entry.split(',') }
          elsif dictionary_entry.interval =~ /TimeStep/
            timestamps = text.scan(/^2,.*/).reject { |entry| entry =~ /^2,.*?,\s0\.00,60\.00,.*/}.map { |entry| entry.split(',') }
          elsif dictionary_entry.interval =~ /Daily/
            timestamps = text.scan(/^3,/).map { |entry| entry.split(',') }
          elsif dictionary_entry.interval =~ /Monthly/
            timestamps = text.scan(/^4,/).map { |entry| entry.split(',') }
          end
          var_data = text.scan(Regexp.new("^#{dictionary_entry.id},.*")).map { |entry| entry.split(',') }
          @variables << Variable.new(dictionary_entry,var_data,timestamps)
        end
      end
      return @variables
    end
  end

end #module
