######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  require 'pathname'

  class ResultsLog
    def initialize(path)
      @path = Pathname.new(path)
      @entries = []
      @initialized = false
      @valid = false
    end

    def find_entries(*search_terms)
      init if not @initialized
      entries = @entries.select do |entry|
        found = true
        search_terms.each do |search_term|
          if not entry[:description] =~ Regexp.new(search_term)
            found = false
          end
        end
        found
      end
      return entries
    end

    def find_entry(*search_terms)
      return find_entries(*search_terms).first
    end

    def entries
      init if not @initialized
      return @entries
    end

    def valid?
      init if not @initialized
      return @valid
    end

    private

    def init
      @initialized = true

      if @path.exist?
        File.open(@path) do |f|
          text = f.read
          @entries = parse_entries(text)
        end
      end

      total_site_energy = find_entry('Total Energy','Total Site Energy')
      if not total_site_energy.nil?
        if total_site_energy[:value].to_f > 0
          @valid = true
        end
      end
    end

    def parse_entries(text)
      entries = []
      lines = text.split("\n")
      lines.each do |line|
        entry = Hash.new
        begin
          entry[:units] = line.match(/\[(.*?)\]/)[1]
        rescue
          entry[:units] = ''
        end
        entry[:description] = line.match(/(.*?),/)[1].sub(/\[.*?\]/,'').strip
        entry[:value] = line.match(/,(.*)/)[1]
        entries << entry
      end
      return entries
    end

  end

end #module