######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

require 'pathname'
require 'csv'
require 'Formats/Formats.rb'

def make_schedule_name(s)
  schedule_name = s
  schedule_name = schedule_name.gsub(/[A-Z][a-z]/, ' \0').lstrip
  schedule_name = schedule_name.gsub(/[a-z][A-Z]/) {|s| "" << s[0] << " " << s[1]}
  schedule_name = schedule_name.gsub('_', ' ').gsub('-',' ').gsub(',',' ').gsub('()', '')
  schedule_name = schedule_name.split(' ').each{|p| p.capitalize!}.join(' ')
  return schedule_name
end

class Schedule
  attr_accessor :name, :type, :table, :hours_per_year

  def initialize(s)
    @lines = s.split("\n")
    @name = make_schedule_name(@lines[0])
    @type = @lines[1]

    @table = Array.new
    this_table = Array.new
    duration = ""
    this_cols = 0
    open = false
    multi_duration = false
    (2..@lines.length-1).each do |idx|
      parts = CSV.parse_line(@lines[idx])

      if parts[0].include?('Through')
        if open
            if (not this_table[-1][0] == 'Total Hours/Year')
              this_table.push(Array.new)
              this_table[-1].push('Total Hours/Year') if not multi_duration
              (1..this_cols-1).each {|i| this_table[-1].push("") }
            end
            multi_duration = true
            close_table(this_table)
        end
        this_table = Array.new
        duration = parts[0]
        open = true

      elsif parts[0]==('Hour')
        this_table.push(Array.new)
        this_cols = parts.length
        this_table[-1].push(parts[0]) if not multi_duration
        (1..this_cols-1).each {|i| this_table[-1].push(parts[i] + " (" + duration + ")") }

      elsif parts[0].include?('Total Hours/Day')
        this_table.push(Array.new)
        this_table[-1].push(parts[0]) if not multi_duration
        (1..this_cols-1).each {|i| this_table[-1].push(custom_print(parts[i].to_f, 4)) }

      elsif parts[0].include?('Total Hours/Week')
        this_table.push(Array.new)
        this_table[-1].push(parts[0]) if not multi_duration
        this_table[-1].push(custom_print(parts[1].to_f, 4))
        (2..this_cols-1).each {|i| this_table[-1].push("") }

      elsif parts[0].include?('Total Hours/Year')
        @hours_per_year = parts[1].to_f

        this_table.push(Array.new)
        this_table[-1].push(parts[0]) if not multi_duration
        this_table[-1].push(custom_print(parts[1].to_f, 4))
        (2..this_cols-1).each {|i| this_table[-1].push("") }
      else
        parts.delete_at(0) if multi_duration
        this_table.push(parts)
      end
    end
    close_table(this_table)
    #print_table if multi_duration
  end

  def close_table(this_table)
    if @table.empty?
      @table = this_table
    else
      @table.each_index do |idx|
        @table[idx].concat(this_table[idx])
      end
    end
  end

  def print_table
    puts @name
    puts @type
    @table.each_index do |idx|
      puts @table[idx].join(', ')
    end
    puts
  end

  def to_s
    @lines.join("\n")
  end
  def hash
    to_s.hash
  end
  def ==(other)
    eql?(other)
  end
  def eql?(other)
    other.is_a?(Schedule) && to_s == other.to_s
  end
end


class SchedulesCSV
  attr_accessor :schedules

  def initialize(path)
  @schedules = Hash.new
    if File.exists?(path)
      @path = Pathname.new(path)
      process_schedules
    else
      @path = nil
    end
  end

  def process_schedules
    File.open(@path) do |f|
      name = ""
      schedule_string = ""
      while line = f.gets

        if line.chomp.empty?
          if not name.empty?
            new_schedule = Schedule.new(schedule_string)
            @schedules[new_schedule.name] = new_schedule
          end
          name = ""
          schedule_string = ""
          next
        end

        if name.empty?
          name = line.chomp
          schedule_string = line
        else
          schedule_string += line
        end
      end
    end
  end
end