######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus
  require 'pathname'

  class SlabGtpFile
    attr_accessor :convergence
    attr_accessor :months
    attr_accessor :averageTemps

    def initialize(path)
      @path = Pathname.new(path)
      @convergence = false
      @months = []
      @averageTemps = []
      init
    end

    def getGroundTemps
      result = "Site:GroundTemperature:BuildingSurface,\n"
      result+= "  #{@averageTemps[0]},  !- January Ground Temperature {C}\n"
      result+= "  #{@averageTemps[1]},  !- February Ground Temperature {C}\n"
      result+= "  #{@averageTemps[2]},  !- March Ground Temperature {C}\n"
      result+= "  #{@averageTemps[3]},  !- April Ground Temperature {C}\n"
      result+= "  #{@averageTemps[4]},  !- May Ground Temperature {C}\n"
      result+= "  #{@averageTemps[5]},  !- June Ground Temperature {C}\n"
      result+= "  #{@averageTemps[6]},  !- July Ground Temperature {C}\n"
      result+= "  #{@averageTemps[7]},  !- August Ground Temperature {C}\n"
      result+= "  #{@averageTemps[8]},  !- September Ground Temperature {C}\n"
      result+= "  #{@averageTemps[9]},  !- October Ground Temperature {C}\n"
      result+= "  #{@averageTemps[10]},  !- November Ground Temperature {C}\n"
      result+= "  #{@averageTemps[11]};  !- December Ground Temperature {C}\n"
      return result
    end

    private

    def init
      if @path.exist?
        File.open(@path) do |f|
          text = f.read
          parse(text)
        end
      end
    end

    def parse(text)
      # check for convergence
      regex = /(Convergence has been gained.).*\n/
      if text.match(regex) then
        @convergence = true
      end

      #parse out the monthly values
      startRegex = /\s*Month\s*TAverage\s*TPerimeter\s*TCore\s*TInside\s*AverageFlux\s*PerimeterFlux\s*CoreFlux/
      readingTable = false
      numRowsRead = 0
      text.each_line do |line|
        if readingTable

          # split each row and count as read
          data = line.strip.split(/\s+/)
          numRowsRead += 1

          # insert into members
          @months << data[2]
          @averageTemps << data[1]

          # only read 12 months
          if numRowsRead == 12
            break
          end
        end

        # have we started reading the table yet
        if line.match(startRegex)
          readingTable = true
        end
      end
    end

  end
end