######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus
  require 'pathname'

  class SlabIdfFile

    def initialize(path)
      @path = Pathname.new(path)
      @initialized = false
      @text = ""
      init
    end

    def savefileas(path)
       File.open(path, "w") do |f|
         f << @text
       end
    end

    # function to alter input based on options
    def setSetpoints(cdd, hdd)
      #options is an array of cdd and hdd... so have to wait the values
      #create a new array with the weighted values
      heatSP = 21  #in deg c
      coolSP = 24  #in deg c

      weight = []
      for i in 1..12
        num = cdd * coolSP + hdd * heatSP
        den = cdd + hdd
        weight[i] = (num) / (den)
      end

      @text.gsub!("%JAN%", sprintf("%.2f", weight[1].to_s))
      @text.gsub!("%FEB%", sprintf("%.2f", weight[2].to_s))
      @text.gsub!("%MAR%", sprintf("%.2f", weight[3].to_s))
      @text.gsub!("%APR%", sprintf("%.2f", weight[4].to_s))
      @text.gsub!("%MAY%", sprintf("%.2f", weight[5].to_s))
      @text.gsub!("%JUN%", sprintf("%.2f", weight[6].to_s))
      @text.gsub!("%JUL%", sprintf("%.2f", weight[7].to_s))
      @text.gsub!("%AUG%", sprintf("%.2f", weight[8].to_s))
      @text.gsub!("%SEP%", sprintf("%.2f", weight[9].to_s))
      @text.gsub!("%OCT%", sprintf("%.2f", weight[10].to_s))
      @text.gsub!("%NOV%", sprintf("%.2f", weight[11].to_s))
      @text.gsub!("%DEC%", sprintf("%.2f", weight[12].to_s))
    end

    private

    def init
      @initialized = true

      if @path.exist?
        File.open(@path) do |f|
          @text = f.read
        end
      end
    end
  end
end #module