######################################################################
#  Copyright (c) 2008-2013, Alliance for Sustainable Energy.
#  All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 2.1 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library; if not, write to the Free Software
#  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
######################################################################

module EnergyPlus

  class WaterMainsFile
    require 'pathname'

    attr_accessor :valid
    attr_accessor :ave_outdoor_db_t
    attr_accessor :max_diff_outdoor_db_t

    def initialize
      @ave_outdoor_db_t = -9999
      @max_diff_outdoor_db_t = -9999
    end

    def valid?
      if @ave_outdoor_db_t != -9999 and @max_diff_outdoor_db_t != -9999
        result = true
      else
        result = false
      end
    end

    def idf_snippet
      result = nil

      if valid?
        result  = "Site:WaterMainsTemperature,\n"
        result += "  CORRELATION,             !- Calculation Method\n"
        result += "  ,                        !- Temperature Schedule Name\n"
        result += "  #{@ave_outdoor_db_t},               !- Annual Average Outdoor Air Temperature {C}\n"
        result += "  #{@max_diff_outdoor_db_t};              !- Maximum Difference In Monthly Average Outdoor Air Temperatures {deltaC}\n"
      end

      return result
    end

    def write_idf_snippet(filename)
      if valid?
        File.delete(filename) if File.exists?(filename)
        File.open(filename, 'w') do |fl|
      #add version identifier to file
        fl << "\nVersion,\n  7.1;                                    ! Version Identifier \n\n"
          fl << idf_snippet
        end
      else
        puts "[WaterMains] ERROR ****** No Temperatures #{filename}"
      end
    end
  end

end #module